import unittest
import os
from zzpy import load_matcher, load_matchers, match_area


class TestCase(unittest.TestCase):
    def test_load_matcher(self):
        def _match(config, area_a, area_b):
            return load_matcher(config)(area_a, area_b)
        self.assertTrue(_match(config={"pattern_a": "A|B|C", "pattern_b": "A|B|C"}, area_a=(
            "A", "B", "C"), area_b=("A", "B", "C")))
        self.assertTrue(_match(config={"pattern_a": ".*|.*|.*", "pattern_b": "{province}|{city}|{district}"}, area_a=(
            "A", "B", "C"), area_b=("A", "B", "C")))
        self.assertFalse(_match(config={"pattern_a": ".*|.*|.*", "pattern_b": "{province}|{city}|{district}"}, area_a=(
            "A", "B", "C"), area_b=("A", "B", "D")))
        self.assertTrue(_match(config={"pattern_a": ".*|.*|.*", "pattern_b": "{province}|{city}|.*"}, area_a=(
            "A", "B", "C"), area_b=("A", "B", "D")))

    def test_match_area(self):
        mathcers = load_matchers(os.path.join(
            "test", "static", "match_config.jsonl"))

        def _match(area_a, area_b):
            return match_area(mathcers, area_a, area_b)

        self.assertTrue(_match(("北京市", "北京市", "东城区"), ("北京市", "北京市", "东城区")))
        self.assertTrue(_match(("北京市", "北京市", "东城区"), ("北京市", "北京市", "西城区")))
        self.assertFalse(_match(("北京市", "北京市", "东城区"), ("北京市", "XXX", "西城区")))
        self.assertTrue(_match(("重庆市", "XXX", "YYY"), ("重庆市", "AAA", "BBB")))



def dict_extract(dic, fields, *args):
    from .zfunction import list_or_args
    fields = list_or_args(fields, args)
    new_dic = {}
    for k, v in dic.items():
        if k in fields:
            new_dic[k] = v
    return new_dic



import unittest
from zzpy import dict_extract


class TestCase(unittest.TestCase):
    def test1(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, "a")
        self.assertDictEqual(d2, {"a": 1})

    def test2(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, "a")
        d2["a"] = 0
        self.assertEqual(d2["a"], 0)
        self.assertEqual(d1["a"], 1)

    def test3(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, "a", "b")
        self.assertDictEqual(d2, {"a": 1, "b": 2})

    def test4(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, ("a", "b"))
        self.assertDictEqual(d2, {"a": 1, "b": 2})

    def test5(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, {"a", "b"})
        self.assertDictEqual(d2, {"a": 1, "b": 2})

    def test6(self):
        d1 = {
            "a": 1, "b": 2, "c": 3
        }
        d2 = dict_extract(d1, ["a", "b"])
        self.assertDictEqual(d2, {"a": 1, "b": 2})



def list_or_args(items, args):
    """COPY FROM redis.client.py"""
    # returns a single new list combining items and args
    try:
        iter(items)
        # a string or bytes instance can be iterated, but indicates
        # items wasn't passed as a list
        if isinstance(items, (str, bytes)):
            items = [items]
        else:
            items = list(items)
    except TypeError:
        items = [items]
    if args:
        items.extend(args)
    return items



def __match(pattern, value):
    import re
    return any(re.match(p, value) for p in pattern.split(","))


def match_area(matchers, area_a, area_b):
    return any(m(area_a, area_b) for m in matchers)


def load_matcher(config):
    def _f(area_a, area_b):
        assert len(area_a) == 3
        assert len(area_b) == 3
        patterns_std = config["pattern_a"].split("|")
        assert len(patterns_std) == 3
        patterns_3rd = config["pattern_b"].split("|")
        assert len(patterns_3rd) == 3
        args = {"province": area_a[0],
                "city": area_a[1], "district": area_a[2]}
        return all(__match(pattern, area) for pattern, area in zip(patterns_std, area_a)) and all(__match(pattern.format(**args), area) for pattern, area in zip(patterns_3rd, area_b))
    return _f


def load_matchers(path):
    import zzpy as z
    return [load_matcher(config) for config in z.read_jsonline(path)]


def main():
    matcher = load_matcher(
        {"pattern_a": "重庆市|.*|.*", "pattern_b": "重庆市|.*|.*"})
    print(matcher(("重庆市", "asdf", "123"), ("重庆市", "xxx", "qqq")))
    
    matcher = load_matcher(
        {"pattern_a": ".*|.*|.*", "pattern_b": "{province}|{city}|.*"})
    print(matcher(("重庆市", "asdf", "123"), ("重庆市", "xxx", "qqq")))
    print(matcher(("重庆市", "asdf", "123"), ("重庆市", "asdf", "qqq")))


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="zzpy",
    version="1.1.19",
    keywords=["pip", "zzpy", "utils"],
    description="Python3 utilities",
    long_description="Python3 utilities",
    license="MIT Licence",

    url="https://github.com/zerocpp/zzpy",
    author="zerocpp",
    author_email="85642468@qq.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["pymongo", "pymysql",
                      "redis", "tqdm", "jsonlines", "pandas", "oss2", "openpyxl", "elasticsearch", "Deprecated", "xlrd"]
)
